#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_ntp_servers_peers
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxNtpServersPeersModule(TestOnyxModule):

    module = onyx_ntp_servers_peers
    enabled = False

    def setUp(self):
        self.enabled = False
        super(TestOnyxNtpServersPeersModule, self).setUp()
        self.mock_get_config = patch.object(
            onyx_ntp_servers_peers.OnyxNTPServersPeersModule, "_show_peers_servers_config")
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestOnyxNtpServersPeersModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        config_file = 'onyx_ntp_servers_peers_show.cfg'
        data = load_fixture(config_file)
        self.get_config.return_value = data
        self.load_config.return_value = None

    def test_ntp_peer_state_no_change(self):
        set_module_args(dict(peer=[dict(ip_or_name='1.1.1.1',
                                        enabled='yes')]))
        self.execute_module(changed=False)

    def test_ntp_peer_state_with_change(self):
        set_module_args(dict(peer=[dict(ip_or_name='1.1.1.1',
                                        enabled='no')]))
        commands = ['ntp peer 1.1.1.1 disable']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_peer_version_no_change(self):
        set_module_args(dict(peer=[dict(ip_or_name='1.1.1.1',
                                        version='4')]))
        self.execute_module(changed=False)

    def test_ntp_peer_version_with_change(self):
        set_module_args(dict(peer=[dict(ip_or_name='1.1.1.1',
                                        version='3')]))
        commands = ['ntp peer 1.1.1.1 version 3']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_peer_key_id_no_change(self):
        set_module_args(dict(peer=[dict(ip_or_name='1.1.1.1',
                                        key_id='5')]))
        self.execute_module(changed=False)

    def test_ntp_peer_key_id_with_change(self):
        set_module_args(dict(peer=[dict(ip_or_name='1.1.1.1',
                                        key_id='6')]))
        commands = ['ntp peer 1.1.1.1 keyID 6']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_peer_delete_with_change(self):
        set_module_args(dict(peer=[dict(ip_or_name='1.1.1.1',
                                        state='absent')]))
        commands = ['no ntp peer 1.1.1.1']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_server_state_no_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          enabled='no')]))
        self.execute_module(changed=False)

    def test_ntp_server_state_with_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          enabled='yes')]))
        commands = ['no ntp server 2.2.2.2 disable']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_server_version_no_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          version='4')]))
        self.execute_module(changed=False)

    def test_ntp_server_version_with_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          version='3')]))
        commands = ['ntp server 2.2.2.2 version 3']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_server_keyID_no_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          key_id='99')]))
        self.execute_module(changed=False)

    def test_ntp_server_keyID_with_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          key_id='8')]))
        commands = ['ntp server 2.2.2.2 keyID 8']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_server_trusted_state_no_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          trusted_enable='yes')]))
        self.execute_module(changed=False)

    def test_ntp_server_trusted_state_with_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          trusted_enable='no')]))
        commands = ['no ntp server 2.2.2.2 trusted-enable']
        self.execute_module(changed=True, commands=commands)

    def test_ntp_server_delete_with_change(self):
        set_module_args(dict(server=[dict(ip_or_name='2.2.2.2',
                                          state='absent')]))
        commands = ['no ntp server 2.2.2.2']
        self.execute_module(changed=True, commands=commands)

    def test_ntpdate_with_change(self):
        set_module_args(dict(ntpdate='192.22.1.66'))
        commands = ['ntpdate 192.22.1.66']
        self.execute_module(changed=True, commands=commands)



#!/usr/bin/python
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: onyx_command
extends_documentation_fragment:
- community.network.onyx

author: "Samer Deeb (@samerd)"
short_description: Run commands on remote devices running Mellanox ONYX
description:
  - Sends arbitrary commands to an Mellanox ONYX network device and returns
    the results read from the device. This module includes an
    argument that will cause the module to wait for a specific condition
    before returning or timing out if the condition is not met.
  - This module does not support running commands in configuration mode.
    Please use M(onyx_config) to configure Mellanox ONYX devices.
notes:
  - Tested on ONYX 3.6.4000
options:
  commands:
    description:
      - List of commands to send to the remote Mellanox ONYX network device.
        The resulting output from the command
        is returned. If the I(wait_for) argument is provided, the
        module is not returned until the condition is satisfied or
        the number of retries has expired.
    required: true
  wait_for:
    description:
      - List of conditions to evaluate against the output of the
        command. The task will wait for each condition to be true
        before moving forward. If the conditional is not true
        within the configured number of retries, the task fails.
        See examples.
  match:
    description:
      - The I(match) argument is used in conjunction with the
        I(wait_for) argument to specify the match policy.  Valid
        values are C(all) or C(any).  If the value is set to C(all)
        then all conditionals in the wait_for must be satisfied.  If
        the value is set to C(any) then only one of the values must be
        satisfied.
    default: all
    choices: ['any', 'all']
  retries:
    description:
      - Specifies the number of retries a command should by tried
        before it is considered failed. The command is run on the
        target device every retry and evaluated against the
        I(wait_for) conditions.
    default: 10
  interval:
    description:
      - Configures the interval in seconds to wait between retries
        of the command. If the command does not pass the specified
        conditions, the interval indicates how long to wait before
        trying the command again.
    default: 1
'''

EXAMPLES = """
tasks:
  - name: Run show version on remote devices
    onyx_command:
      commands: show version

  - name: Run show version and check to see if output contains MLNXOS
    onyx_command:
      commands: show version
      wait_for: result[0] contains MLNXOS

  - name: Run multiple commands on remote nodes
    onyx_command:
      commands:
        - show version
        - show interfaces

  - name: Run multiple commands and evaluate the output
    onyx_command:
      commands:
        - show version
        - show interfaces
      wait_for:
        - result[0] contains MLNXOS
        - result[1] contains mgmt1
"""

RETURN = """
stdout:
  description: The set of responses from the commands
  returned: always apart from low level errors (such as action plugin)
  type: list
  sample: ['...', '...']
stdout_lines:
  description: The value of stdout split into a list
  returned: always apart from low level errors (such as action plugin)
  type: list
  sample: [['...', '...'], ['...'], ['...']]
failed_conditions:
  description: The list of conditionals that have failed
  returned: failed
  type: list
  sample: ['...', '...']
"""

import time

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.parsing import Conditional
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import ComplexList
from ansible.module_utils.six import string_types

from ansible_collections.community.network.plugins.module_utils.network.onyx.onyx import run_commands


def to_lines(stdout):
    for item in stdout:
        if isinstance(item, string_types):
            item = str(item).split('\n')
        yield item


def parse_commands(module, warnings):
    command = ComplexList(dict(
        command=dict(key=True),
        prompt=dict(),
        answer=dict()
    ), module)
    commands = command(module.params['commands'])
    for item in list(commands):
        if module.check_mode and not item['command'].startswith('show'):
            warnings.append(
                'only show commands are supported when using check mode, not '
                'executing `%s`' % item['command']
            )
            commands.remove(item)
        elif item['command'].startswith('conf'):
            module.fail_json(
                msg='onyx_command does not support running config mode '
                    'commands.  Please use onyx_config instead'
            )
    return commands


def main():
    """main entry point for module execution
    """
    argument_spec = dict(
        commands=dict(type='list', required=True),

        wait_for=dict(type='list'),
        match=dict(default='all', choices=['all', 'any']),

        retries=dict(default=10, type='int'),
        interval=dict(default=1, type='int')
    )

    module = AnsibleModule(argument_spec=argument_spec,
                           supports_check_mode=True)

    result = {'changed': False}

    warnings = list()
    commands = parse_commands(module, warnings)
    result['warnings'] = warnings

    wait_for = module.params['wait_for'] or list()
    conditionals = [Conditional(c) for c in wait_for]

    retries = module.params['retries']
    interval = module.params['interval']
    match = module.params['match']

    while retries > 0:
        responses = run_commands(module, commands)

        for item in list(conditionals):
            if item(responses):
                if match == 'any':
                    conditionals = list()
                    break
                conditionals.remove(item)

        if not conditionals:
            break

        time.sleep(interval)
        retries -= 1

    if conditionals:
        failed_conditions = [item.raw for item in conditionals]
        msg = 'One or more conditional statements have not been satisfied'
        module.fail_json(msg=msg, failed_conditions=failed_conditions)

    result.update({
        'changed': False,
        'stdout': responses,
        'stdout_lines': list(to_lines(responses))
    })

    module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c), Gregory Shulov <gregory.shulov@gmail.com>,2016
#
# Simplified BSD License (see licenses/simplified_bsd.txt or https://opensource.org/licenses/BSD-2-Clause)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

HAS_INFINISDK = True
try:
    from infinisdk import InfiniBox, core
except ImportError:
    HAS_INFINISDK = False

from functools import wraps
from os import environ
from os import path


def api_wrapper(func):
    """ Catch API Errors Decorator"""
    @wraps(func)
    def __wrapper(*args, **kwargs):
        module = args[0]
        try:
            return func(*args, **kwargs)
        except core.exceptions.APICommandException as e:
            module.fail_json(msg=e.message)
        except core.exceptions.SystemNotFoundException as e:
            module.fail_json(msg=e.message)
        except Exception:
            raise
    return __wrapper


@api_wrapper
def get_system(module):
    """Return System Object or Fail"""
    box = module.params['system']
    user = module.params.get('user', None)
    password = module.params.get('password', None)

    if user and password:
        system = InfiniBox(box, auth=(user, password))
    elif environ.get('INFINIBOX_USER') and environ.get('INFINIBOX_PASSWORD'):
        system = InfiniBox(box, auth=(environ.get('INFINIBOX_USER'), environ.get('INFINIBOX_PASSWORD')))
    elif path.isfile(path.expanduser('~') + '/.infinidat/infinisdk.ini'):
        system = InfiniBox(box)
    else:
        module.fail_json(msg="You must set INFINIBOX_USER and INFINIBOX_PASSWORD environment variables or set username/password module arguments")

    try:
        system.login()
    except Exception:
        module.fail_json(msg="Infinibox authentication failed. Check your credentials")
    return system


def infinibox_argument_spec():
    """Return standard base dictionary used for the argument_spec argument in AnsibleModule"""

    return dict(
        system=dict(required=True),
        user=dict(),
        password=dict(no_log=True),
    )


def infinibox_required_together():
    """Return the default list used for the required_together argument to AnsibleModule"""
    return [['user', 'password']]



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: vmware_host_ntp_info
short_description: Gathers info about NTP configuration on an ESXi host
description:
- This module can be used to gather information about NTP configurations on an ESXi host.
author:
- Abhijeet Kasurde (@Akasurde)
notes:
- Tested on vSphere 6.5
requirements:
- python >= 2.6
- PyVmomi
options:
  cluster_name:
    description:
    - Name of the cluster.
    - NTP config information about each ESXi server will be returned for the given cluster.
    - If C(esxi_hostname) is not given, this parameter is required.
    type: str
  esxi_hostname:
    description:
    - ESXi hostname.
    - NTP config information about this ESXi server will be returned.
    - If C(cluster_name) is not given, this parameter is required.
    type: str
extends_documentation_fragment:
- community.vmware.vmware.documentation

'''

EXAMPLES = r'''
- name: Gather NTP info about all ESXi Host in the given Cluster
  community.vmware.vmware_host_ntp_info:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    cluster_name: cluster_name
  delegate_to: localhost
  register: cluster_host_ntp

- name: Gather NTP info about ESXi Host
  community.vmware.vmware_host_ntp_info:
    hostname: '{{ vcenter_hostname }}'
    username: '{{ vcenter_username }}'
    password: '{{ vcenter_password }}'
    esxi_hostname: '{{ esxi_hostname }}'
  delegate_to: localhost
  register: host_ntp
'''

RETURN = r'''
hosts_ntp_info:
    description:
    - dict with hostname as key and dict with NTP infos as value
    returned: hosts_ntp_info
    type: dict
    sample: {
        "10.76.33.226": [
            {
                "ntp_servers": [],
                "time_zone_description": "UTC",
                "time_zone_gmt_offset": 0,
                "time_zone_identifier": "UTC",
                "time_zone_name": "UTC"
            }
        ]
    }
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.community.vmware.plugins.module_utils.vmware import vmware_argument_spec, PyVmomi


class VmwareNtpInfoManager(PyVmomi):
    def __init__(self, module):
        super(VmwareNtpInfoManager, self).__init__(module)
        cluster_name = self.params.get('cluster_name', None)
        esxi_host_name = self.params.get('esxi_hostname', None)
        self.hosts = self.get_all_host_objs(cluster_name=cluster_name, esxi_host_name=esxi_host_name)

    def gather_ntp_info(self):
        hosts_info = {}
        for host in self.hosts:
            host_ntp_info = []
            host_date_time_manager = host.configManager.dateTimeSystem
            if host_date_time_manager:
                host_ntp_info.append(
                    dict(
                        time_zone_identifier=host_date_time_manager.dateTimeInfo.timeZone.key,
                        time_zone_name=host_date_time_manager.dateTimeInfo.timeZone.name,
                        time_zone_description=host_date_time_manager.dateTimeInfo.timeZone.description,
                        time_zone_gmt_offset=host_date_time_manager.dateTimeInfo.timeZone.gmtOffset,
                        ntp_servers=[ntp_server for ntp_server in host_date_time_manager.dateTimeInfo.ntpConfig.server]
                    )
                )
            hosts_info[host.name] = host_ntp_info
        return hosts_info


def main():
    argument_spec = vmware_argument_spec()
    argument_spec.update(
        cluster_name=dict(type='str', required=False),
        esxi_hostname=dict(type='str', required=False),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=[
            ['cluster_name', 'esxi_hostname'],
        ],
        supports_check_mode=True,
    )

    vmware_host_ntp_config = VmwareNtpInfoManager(module)
    module.exit_json(changed=False, hosts_ntp_info=vmware_host_ntp_config.gather_ntp_info())


if __name__ == "__main__":
    main()



#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible_collections.community.network.tests.unit.compat.mock import patch
from ansible_collections.community.network.plugins.modules.network.onyx import onyx_syslog_files
from ansible_collections.community.network.tests.unit.plugins.modules.utils import set_module_args
from .onyx_module import TestOnyxModule, load_fixture


class TestOnyxSyslogFilesModule(TestOnyxModule):

    module = onyx_syslog_files

    def setUp(self):
        self.enabled = False
        super(TestOnyxSyslogFilesModule, self).setUp()
        self.mock_get_config = patch.object(
            onyx_syslog_files.OnyxSyslogFilesModule, "show_logging")
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch(
            'ansible_collections.community.network.plugins.module_utils.network.onyx.onyx.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestOnyxSyslogFilesModule, self).tearDown()
        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        config_file = 'onyx_logging_show.cfg'
        self.get_config.return_value = load_fixture(config_file)
        self.load_config.return_value = None

    def test_syslog_files_force_rotate(self):
        set_module_args(dict(rotation=dict(force=True)))
        commands = ["logging files rotation force"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_files_max_num(self):
        set_module_args(dict(rotation=dict(max_num=30)))
        commands = ["logging files rotation max-num 30"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_files_freq(self):
        set_module_args(dict(rotation=dict(frequency="daily")))
        commands = ["logging files rotation criteria frequency daily"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_files_size(self):
        set_module_args(dict(rotation=dict(size=10.5)))
        commands = ["logging files rotation criteria size 10.5"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_files_delete(self):
        set_module_args(dict(delete_group="oldest"))
        commands = ["logging files delete oldest"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_debug_files_force_rotate(self):
        set_module_args(dict(rotation=dict(force=True), debug=True))
        commands = ["logging debug-files rotation force"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_debug_files_max_num(self):
        set_module_args(dict(rotation=dict(max_num=30), debug=True))
        commands = ["logging debug-files rotation max-num 30"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_debug_files_freq(self):
        set_module_args(dict(rotation=dict(frequency="weekly"), debug=True))
        commands = ["logging debug-files rotation criteria frequency weekly"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_debug_files_size(self):
        set_module_args(dict(rotation=dict(size=10.5), debug=True))
        commands = ["logging debug-files rotation criteria size 10.5"]
        self.execute_module(changed=True, commands=commands)

    def test_syslog_debug_files_delete(self):
        set_module_args(dict(delete_group="oldest", debug=True))
        commands = ["logging debug-files delete oldest"]
        self.execute_module(changed=True, commands=commands)

    ''' nochange '''
    def test_syslog_files_max_num_no_change(self):
        set_module_args(dict(rotation=dict(max_num=10)))
        self.execute_module(changed=False)

    def test_syslog_files_freq_no_change(self):
        set_module_args(dict(rotation=dict(frequency="weekly")))
        self.execute_module(changed=False)

    def test_syslog_files_size_no_change(self):
        set_module_args(dict(rotation=dict(size_pct=10)))
        self.execute_module(changed=False)

    def test_syslog_debug_files_max_num_no_change(self):
        set_module_args(dict(rotation=dict(max_num=20), debug=True))
        self.execute_module(changed=False)

    def test_syslog_debug_files_freq_no_change(self):
        set_module_args(dict(rotation=dict(frequency="daily"), debug=True))
        self.execute_module(changed=False)

    def test_syslog_debug_files_size_no_change(self):
        set_module_args(dict(rotation=dict(size=20), debug=True))
        self.execute_module(changed=False)
